@@ -1,10 +1,10 @@
 // Copyright (C) 2021 Mathieu Duponchelle <mathieu@centricular.com>
 //
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
+//
+// SPDX-License-Identifier: MPL-2.0
 
 // TODO:
 //
@@ -38,6 +38,9 @@ use atomic_refcell::AtomicRefCell;
 use once_cell::sync::Lazy;
 
 use std::collections::BTreeMap;
+use std::sync::Mutex;
+
+const DEFAULT_UNBUFFERED: bool = false;
 
 #[derive(Debug)]
 struct TimestampedLines {
@@ -178,6 +181,19 @@ impl From<Row> for Line {
     }
 }
 
+#[derive(Clone)]
+struct Settings {
+    unbuffered: bool,
+}
+
+impl Default for Settings {
+    fn default() -> Self {
+        Self {
+            unbuffered: DEFAULT_UNBUFFERED,
+        }
+    }
+}
+
 struct State {
     mode: Option<Cea608Mode>,
     last_cc_data: Option<u16>,
@@ -189,6 +205,7 @@ struct State {
     clear: Option<bool>,
     cursor: Cursor,
     pending_lines: Option<TimestampedLines>,
+    settings: Settings,
 }
 
 impl Default for State {
@@ -209,6 +226,7 @@ impl Default for State {
                 underline: false,
             },
             pending_lines: None,
+            settings: Settings::default(),
         }
     }
 }
@@ -218,6 +236,7 @@ pub struct Cea608ToJson {
     sinkpad: gst::Pad,
 
     state: AtomicRefCell<State>,
+    settings: Mutex<Settings>,
 }
 
 static CAT: Lazy<gst::DebugCategory> = Lazy::new(|| {
@@ -368,8 +387,8 @@ fn eia608_to_text(cc_data: u16) -> String {
     unsafe {
         let bufsz = ffi::eia608_to_text(std::ptr::null_mut(), 0, cc_data);
         let mut data = Vec::with_capacity((bufsz + 1) as usize);
-        data.set_len(bufsz as usize);
         ffi::eia608_to_text(data.as_ptr() as *mut _, (bufsz + 1) as usize, cc_data);
+        data.set_len(bufsz as usize);
         String::from_utf8_unchecked(data)
     }
 }
@@ -381,9 +400,7 @@ fn dump(
     duration: impl Into<Option<gst::ClockTime>>,
 ) {
     let pts = pts.into();
-    let end = pts
-        .zip(duration.into())
-        .map(|(pts, duration)| pts + duration);
+    let end = pts.opt_add(duration.into());
 
     if cc_data != 0x8080 {
         gst_debug!(
@@ -413,14 +430,14 @@ impl State {
     ) -> Option<TimestampedLines> {
         if mode.is_rollup() && self.mode == Some(Cea608Mode::PopOn) {
             // https://www.law.cornell.edu/cfr/text/47/79.101 (f)(2)(v)
-            let _ = self.drain(element);
+            let _ = self.drain(element, true);
         }
 
         let ret = if Some(mode) != self.mode {
             if self.mode == Some(Cea608Mode::PopOn) {
                 self.drain_pending(element)
             } else {
-                self.drain(element)
+                self.drain(element, true)
             }
         } else {
             None
@@ -443,36 +460,53 @@ impl State {
         ret
     }
 
-    fn drain(&mut self, element: &super::Cea608ToJson) -> Option<TimestampedLines> {
+    fn drain(&mut self, element: &super::Cea608ToJson, flush: bool) -> Option<TimestampedLines> {
         gst_log!(CAT, obj: element, "Draining");
 
-        let pts = self.first_pts;
+        let pts = if self.settings.unbuffered {
+            self.current_pts
+        } else {
+            self.first_pts
+        };
 
-        let duration = match self.mode {
-            Some(Cea608Mode::PopOn) => gst::ClockTime::NONE,
-            _ => self
-                .current_pts
-                .zip(self.current_duration)
-                .map(|(cur_pts, cur_duration)| cur_pts + cur_duration)
-                .zip(self.first_pts)
-                .and_then(|(cur_end, first_pts)| cur_end.checked_sub(first_pts)),
+        let duration = if self.settings.unbuffered {
+            self.current_duration
+        } else {
+            match self.mode {
+                Some(Cea608Mode::PopOn) => gst::ClockTime::NONE,
+                _ => self
+                    .current_pts
+                    .opt_add(self.current_duration)
+                    .opt_checked_sub(self.first_pts)
+                    .ok()
+                    .flatten(),
+            }
         };
 
         self.first_pts = gst::ClockTime::NONE;
 
         let mut lines: Vec<Line> = vec![];
 
-        // Wish BTreeMap had a drain() method
-        for (_idx, row) in std::mem::take(&mut self.rows).into_iter() {
-            if !row.is_empty() {
-                let mut line: Line = row.into();
-                line.carriage_return = self.carriage_return.take();
-                lines.push(line);
+        if flush {
+            for (_idx, row) in std::mem::take(&mut self.rows).into_iter() {
+                if !row.is_empty() {
+                    let mut line: Line = row.into();
+                    line.carriage_return = self.carriage_return.take();
+                    lines.push(line);
+                }
+            }
+
+            self.rows.clear();
+        } else {
+            for row in self.rows.values() {
+                if !row.is_empty() {
+                    let mut line: Line = row.clone().into();
+                    line.carriage_return = self.carriage_return.take();
+                    lines.push(line);
+                }
             }
         }
 
-        self.rows.clear();
-
         let clear = self.clear.take();
 
         if !lines.is_empty() {
@@ -505,10 +539,10 @@ impl State {
             gst_log!(CAT, obj: element, "Draining pending");
             pending.duration = self
                 .current_pts
-                .zip(self.current_duration)
-                .map(|(cur_pts, cur_dur)| cur_pts + cur_dur)
-                .zip(pending.pts)
-                .and_then(|(cur_end, pending_pts)| cur_end.checked_sub(pending_pts));
+                .opt_add(self.current_duration)
+                .opt_checked_sub(pending.pts)
+                .ok()
+                .flatten();
             Some(pending)
         } else {
             None
@@ -528,6 +562,37 @@ impl State {
 
         gst_log!(CAT, obj: element, "preamble: {:?}", preamble);
 
+        let drain_roll_up = self.cursor.row != preamble.row as u32;
+
+        // In unbuffered mode, we output the whole roll-up window
+        // and need to move it when the preamble relocates it
+        // https://www.law.cornell.edu/cfr/text/47/79.101 (f)(1)(ii)
+        if self.settings.unbuffered {
+            if let Some(mode) = self.mode {
+                if mode.is_rollup() && self.cursor.row != preamble.row as u32 {
+                    let offset = match mode {
+                        Cea608Mode::RollUp2 => 1,
+                        Cea608Mode::RollUp3 => 2,
+                        Cea608Mode::RollUp4 => 3,
+                        _ => unreachable!(),
+                    };
+
+                    let current_top_row = self.cursor.row.saturating_sub(offset);
+                    let new_row_offset = preamble.row - self.cursor.row as i32;
+
+                    for row in current_top_row..self.cursor.row {
+                        if let Some(mut row) = self.rows.remove(&row) {
+                            let new_row = row.row as i32 + new_row_offset;
+                            if new_row >= 0 {
+                                row.row = new_row as u32;
+                                self.rows.insert(row.row, row);
+                            }
+                        }
+                    }
+                }
+            }
+        }
+
         self.cursor.row = preamble.row as u32;
         self.cursor.col = preamble.col as usize;
         self.cursor.style = preamble.style;
@@ -535,22 +600,37 @@ impl State {
 
         if let Some(mode) = self.mode {
             match mode {
-                // The relocation is potentially destructive, let us drain
                 Cea608Mode::RollUp2
                 | Cea608Mode::RollUp3
                 | Cea608Mode::RollUp4
                 | Cea608Mode::PaintOn => {
-                    let ret = self.drain(element);
+                    if self.settings.unbuffered {
+                        /* We only need to drain when the roll-up window was relocated */
+                        let ret = if drain_roll_up {
+                            self.drain(element, true)
+                        } else {
+                            None
+                        };
 
-                    self.rows.insert(self.cursor.row, Row::new(self.cursor.row));
+                        if let std::collections::btree_map::Entry::Vacant(e) =
+                            self.rows.entry(self.cursor.row)
+                        {
+                            e.insert(Row::new(self.cursor.row));
+                        }
 
-                    ret
+                        ret
+                    // The relocation is potentially destructive, let us drain
+                    } else {
+                        let ret = self.drain(element, true);
+                        self.rows.insert(self.cursor.row, Row::new(self.cursor.row));
+
+                        ret
+                    }
                 }
                 Cea608Mode::PopOn => {
-                    #[allow(clippy::map_entry)]
-                    if !self.rows.contains_key(&self.cursor.row) {
-                        self.rows.insert(self.cursor.row, Row::new(self.cursor.row));
-                    }
+                    let row = self.cursor.row;
+                    self.rows.entry(row).or_insert_with(|| Row::new(row));
+
                     None
                 }
             }
@@ -583,7 +663,7 @@ impl State {
                         self.drain_pending(element)
                     }
                     _ => {
-                        let ret = self.drain(element);
+                        let ret = self.drain(element, true);
                         self.clear = Some(true);
                         ret
                     }
@@ -604,8 +684,34 @@ impl State {
                 if let Some(mode) = self.mode {
                     // https://www.law.cornell.edu/cfr/text/47/79.101 (f)(2)(i) (f)(3)(i)
                     if mode.is_rollup() {
-                        let ret = self.drain(element);
-                        self.carriage_return = Some(true);
+                        let ret = if self.settings.unbuffered {
+                            let offset = match mode {
+                                Cea608Mode::RollUp2 => 1,
+                                Cea608Mode::RollUp3 => 2,
+                                Cea608Mode::RollUp4 => 3,
+                                _ => unreachable!(),
+                            };
+
+                            let top_row = self.cursor.row.saturating_sub(offset);
+
+                            // https://www.law.cornell.edu/cfr/text/47/79.101 (f)(1)(iii)
+                            self.rows.remove(&top_row);
+
+                            for row in top_row + 1..self.cursor.row + 1 {
+                                if let Some(mut row) = self.rows.remove(&row) {
+                                    row.row -= 1;
+                                    self.rows.insert(row.row, row);
+                                }
+                            }
+
+                            self.rows.insert(self.cursor.row, Row::new(self.cursor.row));
+                            self.drain(element, false)
+                        } else {
+                            let ret = self.drain(element, true);
+                            self.carriage_return = Some(true);
+                            ret
+                        };
+
                         return ret;
                     }
                 }
@@ -627,8 +733,13 @@ impl State {
                 // https://www.law.cornell.edu/cfr/text/47/79.101 (f)(2)
                 self.update_mode(element, Cea608Mode::PopOn);
                 self.first_pts = self.current_pts;
-                let ret = self.drain_pending(element);
-                self.pending_lines = self.drain(element);
+                let ret = if self.settings.unbuffered {
+                    self.drain(element, true)
+                } else {
+                    let ret = self.drain_pending(element);
+                    self.pending_lines = self.drain(element, true);
+                    ret
+                };
                 return ret;
             }
             ffi::eia608_control_t_eia608_tab_offset_0
@@ -715,9 +826,15 @@ impl State {
             gst_log!(CAT, obj: element, "control!");
             return self.decode_control(element, cc_data);
         } else if is_basicna(cc_data) || is_specialna(cc_data) || is_westeu(cc_data) {
-            self.mode?;
-            gst_log!(CAT, obj: element, "text");
-            self.decode_text(element, cc_data);
+            if let Some(mode) = self.mode {
+                self.mode?;
+                gst_log!(CAT, obj: element, "text");
+                self.decode_text(element, cc_data);
+
+                if mode.is_rollup() && self.settings.unbuffered {
+                    return self.drain(element, false);
+                }
+            }
         } else if is_preamble(cc_data) {
             gst_log!(CAT, obj: element, "preamble");
             return self.decode_preamble(element, cc_data);
@@ -800,6 +917,14 @@ impl Cea608ToJson {
         if let Some(lines) = state.handle_cc_data(element, pts, duration, cc_data) {
             drop(state);
             self.output(element, lines)
+        } else if state.settings.unbuffered {
+            drop(state);
+            self.srcpad.push_event(
+                gst::event::Gap::builder(pts.unwrap())
+                    .duration(duration)
+                    .build(),
+            );
+            Ok(gst::FlowSuccess::Ok)
         } else {
             Ok(gst::FlowSuccess::Ok)
         }
@@ -813,13 +938,15 @@ impl Cea608ToJson {
             EventView::Caps(..) => {
                 // We send our own caps downstream
                 let caps = gst::Caps::builder("application/x-json")
-                    .field("format", &"cea608")
+                    .field("format", "cea608")
                     .build();
                 self.srcpad.push_event(gst::event::Caps::new(&caps))
             }
             EventView::FlushStop(..) => {
                 let mut state = self.state.borrow_mut();
+                let old_settings = state.settings.clone();
                 *state = State::default();
+                state.settings = old_settings;
                 drop(state);
                 pad.event_default(Some(element), event)
             }
@@ -827,7 +954,7 @@ impl Cea608ToJson {
                 if let Some(lines) = self.state.borrow_mut().drain_pending(element) {
                     let _ = self.output(element, lines);
                 }
-                if let Some(lines) = self.state.borrow_mut().drain(element) {
+                if let Some(lines) = self.state.borrow_mut().drain(element, true) {
                     let _ = self.output(element, lines);
                 }
 
@@ -873,6 +1000,7 @@ impl ObjectSubclass for Cea608ToJson {
             srcpad,
             sinkpad,
             state: AtomicRefCell::new(State::default()),
+            settings: Mutex::new(Settings::default()),
         }
     }
 }
@@ -884,8 +1012,51 @@ impl ObjectImpl for Cea608ToJson {
         obj.add_pad(&self.sinkpad).unwrap();
         obj.add_pad(&self.srcpad).unwrap();
     }
+
+    fn properties() -> &'static [glib::ParamSpec] {
+        static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
+            vec![glib::ParamSpecBoolean::new(
+                "unbuffered",
+                "Unbuffered",
+                "Whether captions should be output at display time, \
+                     instead of waiting to determine durations. Useful with live input",
+                DEFAULT_UNBUFFERED,
+                glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
+            )]
+        });
+
+        PROPERTIES.as_ref()
+    }
+
+    fn set_property(
+        &self,
+        _obj: &Self::Type,
+        _id: usize,
+        value: &glib::Value,
+        pspec: &glib::ParamSpec,
+    ) {
+        match pspec.name() {
+            "unbuffered" => {
+                self.settings.lock().unwrap().unbuffered =
+                    value.get().expect("type checked upstream");
+            }
+            _ => unimplemented!(),
+        }
+    }
+
+    fn property(&self, _obj: &Self::Type, _id: usize, pspec: &glib::ParamSpec) -> glib::Value {
+        match pspec.name() {
+            "unbuffered" => {
+                let settings = self.settings.lock().unwrap();
+                settings.unbuffered.to_value()
+            }
+            _ => unimplemented!(),
+        }
+    }
 }
 
+impl GstObjectImpl for Cea608ToJson {}
+
 impl ElementImpl for Cea608ToJson {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -913,7 +1084,7 @@ impl ElementImpl for Cea608ToJson {
             .unwrap();
 
             let caps = gst::Caps::builder("closedcaption/x-cea-608")
-                .field("format", &"raw")
+                .field("format", "raw")
                 .build();
 
             let sink_pad_template = gst::PadTemplate::new(
@@ -930,6 +1101,7 @@ impl ElementImpl for Cea608ToJson {
         PAD_TEMPLATES.as_ref()
     }
 
+    #[allow(clippy::single_match)]
     fn change_state(
         &self,
         element: &Self::Type,
@@ -941,6 +1113,7 @@ impl ElementImpl for Cea608ToJson {
             gst::StateChange::ReadyToPaused => {
                 let mut state = self.state.borrow_mut();
                 *state = State::default();
+                state.settings = self.settings.lock().unwrap().clone();
             }
             _ => (),
         }
