@@ -1,19 +1,10 @@
 // Copyright (C) 2018 Sebastian Dröge <sebastian@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -52,21 +43,12 @@ impl Default for State {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Default, Debug, Clone)]
 struct Settings {
     uuid: Option<String>,
     creation_date: Option<glib::DateTime>,
 }
 
-impl Default for Settings {
-    fn default() -> Self {
-        Self {
-            uuid: None,
-            creation_date: None,
-        }
-    }
-}
-
 pub struct MccEnc {
     srcpad: gst::Pad,
     sinkpad: gst::Pad,
@@ -130,7 +112,7 @@ impl MccEnc {
                     creation_date.seconds() as u32,
                 )
                 .with_timezone(&FixedOffset::east(
-                    (creation_date.utc_offset() / 1_000_000) as i32,
+                    creation_date.utc_offset().as_seconds() as i32
                 ));
 
             let _ = write!(
@@ -157,11 +139,11 @@ impl MccEnc {
             );
         }
 
-        if *framerate.denom() == 1 {
-            let _ = write!(buffer, "Time Code Rate={}\r\n", *framerate.numer());
+        if framerate.denom() == 1 {
+            let _ = write!(buffer, "Time Code Rate={}\r\n", framerate.numer());
         } else {
-            assert_eq!(*framerate.denom(), 1001);
-            let _ = write!(buffer, "Time Code Rate={}DF\r\n", *framerate.numer() / 1000);
+            assert_eq!(framerate.denom(), 1001);
+            let _ = write!(buffer, "Time Code Rate={}DF\r\n", framerate.numer() / 1000);
         }
         let _ = write!(buffer, "\r\n");
 
@@ -388,9 +370,9 @@ impl MccEnc {
                     .field(
                         "version",
                         if framerate == gst::Fraction::new(60000, 1001) {
-                            &2i32
+                            2i32
                         } else {
-                            &1i32
+                            1i32
                         },
                     )
                     .build();
@@ -495,14 +477,14 @@ impl ObjectImpl for MccEnc {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
             vec![
-                glib::ParamSpec::new_string(
+                glib::ParamSpecString::new(
                     "uuid",
                     "UUID",
                     "UUID for the output file",
                     None,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_boxed(
+                glib::ParamSpecBoxed::new(
                     "creation-date",
                     "Creation Date",
                     "Creation date for the output file",
@@ -557,6 +539,8 @@ impl ObjectImpl for MccEnc {
     }
 }
 
+impl GstObjectImpl for MccEnc {}
+
 impl ElementImpl for MccEnc {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -577,24 +561,24 @@ impl ElementImpl for MccEnc {
             {
                 let caps = caps.get_mut().unwrap();
 
-                let framerates = gst::List::new(&[
-                    &gst::Fraction::new(24, 1),
-                    &gst::Fraction::new(25, 1),
-                    &gst::Fraction::new(30000, 1001),
-                    &gst::Fraction::new(30, 1),
-                    &gst::Fraction::new(50, 1),
-                    &gst::Fraction::new(60000, 1001),
-                    &gst::Fraction::new(60, 1),
+                let framerates = gst::List::new([
+                    gst::Fraction::new(24, 1),
+                    gst::Fraction::new(25, 1),
+                    gst::Fraction::new(30000, 1001),
+                    gst::Fraction::new(30, 1),
+                    gst::Fraction::new(50, 1),
+                    gst::Fraction::new(60000, 1001),
+                    gst::Fraction::new(60, 1),
                 ]);
 
                 let s = gst::Structure::builder("closedcaption/x-cea-708")
-                    .field("format", &"cdp")
+                    .field("format", "cdp")
                     .field("framerate", &framerates)
                     .build();
                 caps.append_structure(s);
 
                 let s = gst::Structure::builder("closedcaption/x-cea-608")
-                    .field("format", &"s334-1a")
+                    .field("format", "s334-1a")
                     .field("framerate", &framerates)
                     .build();
                 caps.append_structure(s);
