@@ -1,19 +1,10 @@
 // Copyright (C) 2018 Sebastian Dröge <sebastian@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -25,7 +16,6 @@ use gst::{
 use gst_video::ValidVideoTimeCode;
 
 use std::cmp;
-use std::convert::TryInto;
 use std::sync::{Mutex, MutexGuard};
 
 use once_cell::sync::Lazy;
@@ -274,8 +264,7 @@ impl State {
         }
 
         buffer.set_duration(
-            gst::ClockTime::SECOND
-                .mul_div_ceil(*framerate.denom() as u64, *framerate.numer() as u64),
+            gst::ClockTime::SECOND.mul_div_ceil(framerate.denom() as u64, framerate.numer() as u64),
         );
     }
 
@@ -311,12 +300,12 @@ impl State {
 
                 let caps = match format {
                     Format::Cea708Cdp => gst::Caps::builder("closedcaption/x-cea-708")
-                        .field("format", &"cdp")
-                        .field("framerate", &framerate)
+                        .field("format", "cdp")
+                        .field("framerate", framerate)
                         .build(),
                     Format::Cea608 => gst::Caps::builder("closedcaption/x-cea-608")
-                        .field("format", &"s334-1a")
-                        .field("framerate", &framerate)
+                        .field("format", "s334-1a")
+                        .field("framerate", framerate)
                         .build(),
                 };
 
@@ -375,8 +364,7 @@ impl MccParse {
     ) -> Result<gst::FlowSuccess, gst::FlowError> {
         let mut state = self.state.lock().unwrap();
 
-        let drain;
-        if let Some(buffer) = buffer {
+        let drain = if let Some(buffer) = buffer {
             let buffer = buffer.into_mapped_buffer_readable().map_err(|_| {
                 element_error!(
                     element,
@@ -388,10 +376,10 @@ impl MccParse {
             })?;
 
             state.reader.push(buffer);
-            drain = false;
+            false
         } else {
-            drain = true;
-        }
+            true
+        };
 
         loop {
             let line = state.line(drain);
@@ -955,7 +943,11 @@ impl MccParse {
         }
     }
 
-    fn perform_seek(&self, event: &gst::event::Seek, element: &super::MccParse) -> bool {
+    fn perform_seek(
+        &self,
+        event: &gst::event::Seek<&gst::EventRef>,
+        element: &super::MccParse,
+    ) -> bool {
         if self.state.lock().unwrap().pull.is_none() {
             gst_error!(CAT, obj: element, "seeking is only supported in pull mode");
             return false;
@@ -1011,17 +1003,11 @@ impl MccParse {
         let pull = state.pull.as_ref().unwrap();
 
         if start_type == gst::SeekType::Set {
-            start = start
-                .zip(pull.duration)
-                .map(|(start, duration)| start.min(duration))
-                .or(start);
+            start = start.opt_min(pull.duration).or(start);
         }
 
         if stop_type == gst::SeekType::Set {
-            stop = stop
-                .zip(pull.duration)
-                .map(|(stop, duration)| stop.min(duration))
-                .or(stop);
+            stop = stop.opt_min(pull.duration).or(stop);
         }
 
         state.seeking = true;
@@ -1202,6 +1188,8 @@ impl ObjectImpl for MccParse {
     }
 }
 
+impl GstObjectImpl for MccParse {}
+
 impl ElementImpl for MccParse {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -1227,14 +1215,14 @@ impl ElementImpl for MccParse {
                 );
 
                 let s = gst::Structure::builder("closedcaption/x-cea-708")
-                    .field("format", &"cdp")
-                    .field("framerate", &framerate)
+                    .field("format", "cdp")
+                    .field("framerate", framerate)
                     .build();
                 caps.append_structure(s);
 
                 let s = gst::Structure::builder("closedcaption/x-cea-608")
-                    .field("format", &"s334-1a")
-                    .field("framerate", &framerate)
+                    .field("format", "s334-1a")
+                    .field("framerate", framerate)
                     .build();
                 caps.append_structure(s);
             }
@@ -1247,7 +1235,7 @@ impl ElementImpl for MccParse {
             .unwrap();
 
             let caps = gst::Caps::builder("application/x-mcc")
-                .field("version", &gst::List::new(&[&1i32, &2i32]))
+                .field("version", gst::List::new([1i32, 2i32]))
                 .build();
             let sink_pad_template = gst::PadTemplate::new(
                 "sink",
