@@ -1,20 +1,11 @@
 // Copyright (C) 2019 Sebastian Dröge <sebastian@centricular.com>
 // Copyright (C) 2019 Jordan Petridis <jordan@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -25,7 +16,6 @@ use gst::{
 };
 
 use std::cmp;
-use std::convert::TryInto;
 use std::sync::{Mutex, MutexGuard};
 
 use once_cell::sync::Lazy;
@@ -106,8 +96,6 @@ fn parse_timecode(
     framerate: gst::Fraction,
     tc: &TimeCode,
 ) -> Result<gst_video::ValidVideoTimeCode, gst::FlowError> {
-    use std::convert::TryFrom;
-
     let mut tc = tc.clone();
     // Workaround for various SCC files having invalid drop frame timecodes:
     // Every full minute the first two timecodes are skipped, except for every tenth minute.
@@ -228,8 +216,7 @@ impl State {
 
         buffer.set_pts(self.last_position);
         buffer.set_duration(
-            gst::ClockTime::SECOND
-                .mul_div_ceil(*framerate.denom() as u64, *framerate.numer() as u64),
+            gst::ClockTime::SECOND.mul_div_ceil(framerate.denom() as u64, framerate.numer() as u64),
         );
     }
 
@@ -263,8 +250,8 @@ impl State {
                 self.framerate = Some(framerate);
 
                 let caps = gst::Caps::builder("closedcaption/x-cea-608")
-                    .field("format", &"raw")
-                    .field("framerate", &framerate)
+                    .field("format", "raw")
+                    .field("framerate", framerate)
                     .build();
                 self.framerate = Some(framerate);
 
@@ -303,8 +290,7 @@ impl SccParse {
     ) -> Result<gst::FlowSuccess, gst::FlowError> {
         let mut state = self.state.lock().unwrap();
 
-        let drain;
-        if let Some(buffer) = buffer {
+        let drain = if let Some(buffer) = buffer {
             let buffer = buffer.into_mapped_buffer_readable().map_err(|_| {
                 element_error!(
                     element,
@@ -316,10 +302,10 @@ impl SccParse {
             })?;
 
             state.reader.push(buffer);
-            drain = false;
+            false
         } else {
-            drain = true;
-        }
+            true
+        };
 
         loop {
             let line = state.line(drain);
@@ -429,14 +415,8 @@ impl SccParse {
             timecode.increment_frame();
 
             if clip_buffers {
-                let end_time = buffer
-                    .pts()
-                    .zip(buffer.duration())
-                    .map(|(pts, duration)| pts + duration);
-                if end_time
-                    .zip(segment_start)
-                    .map_or(false, |(end_time, segment_start)| end_time < segment_start)
-                {
+                let end_time = buffer.pts().opt_add(buffer.duration());
+                if end_time.opt_lt(segment_start).unwrap_or(false) {
                     gst_trace!(
                         CAT,
                         obj: element,
@@ -448,12 +428,11 @@ impl SccParse {
                 }
             }
 
-            send_eos = state.segment.stop().map_or(false, |stop| {
-                buffer
-                    .pts()
-                    .zip(buffer.duration())
-                    .map_or(false, |(pts, duration)| pts + duration >= stop)
-            });
+            send_eos = buffer
+                .pts()
+                .opt_add(buffer.duration())
+                .opt_ge(state.segment.stop())
+                .unwrap_or(false);
 
             let buffers = buffers.get_mut().unwrap();
             buffers.add(buffer);
@@ -839,7 +818,11 @@ impl SccParse {
         }
     }
 
-    fn perform_seek(&self, event: &gst::event::Seek, element: &super::SccParse) -> bool {
+    fn perform_seek(
+        &self,
+        event: &gst::event::Seek<&gst::EventRef>,
+        element: &super::SccParse,
+    ) -> bool {
         if self.state.lock().unwrap().pull.is_none() {
             gst_error!(CAT, obj: element, "seeking is only supported in pull mode");
             return false;
@@ -895,17 +878,11 @@ impl SccParse {
         let pull = state.pull.as_ref().unwrap();
 
         if start_type == gst::SeekType::Set {
-            start = start
-                .zip(pull.duration)
-                .map(|(start, duration)| start.min(duration))
-                .or(start);
+            start = start.opt_min(pull.duration).or(start);
         }
 
         if stop_type == gst::SeekType::Set {
-            stop = stop
-                .zip(pull.duration)
-                .map(|(stop, duration)| stop.min(duration))
-                .or(stop);
+            stop = stop.opt_min(pull.duration).or(stop);
         }
 
         state.seeking = true;
@@ -1086,6 +1063,8 @@ impl ObjectImpl for SccParse {
     }
 }
 
+impl GstObjectImpl for SccParse {}
+
 impl ElementImpl for SccParse {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -1103,13 +1082,10 @@ impl ElementImpl for SccParse {
     fn pad_templates() -> &'static [gst::PadTemplate] {
         static PAD_TEMPLATES: Lazy<Vec<gst::PadTemplate>> = Lazy::new(|| {
             let caps = gst::Caps::builder("closedcaption/x-cea-608")
-                .field("format", &"raw")
+                .field("format", "raw")
                 .field(
                     "framerate",
-                    &gst::List::new(&[
-                        &gst::Fraction::new(30000, 1001),
-                        &gst::Fraction::new(30, 1),
-                    ]),
+                    gst::List::new([gst::Fraction::new(30000, 1001), gst::Fraction::new(30, 1)]),
                 )
                 .build();
             let src_pad_template = gst::PadTemplate::new(
