@@ -1,19 +1,10 @@
 // Copyright (C) 2021 Mathieu Duponchelle <mathieu@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use crate::ttutils::Cea608Mode;
 use anyhow::Error;
@@ -67,7 +58,7 @@ impl Default for Settings {
     fn default() -> Self {
         Self {
             cc_caps: gst::Caps::builder("closedcaption/x-cea-608")
-                .field("format", &"raw")
+                .field("format", "raw")
                 .build(),
             passthrough: DEFAULT_PASSTHROUGH,
             latency: DEFAULT_LATENCY,
@@ -139,16 +130,12 @@ impl TranscriberBin {
 
         state
             .transcriber_queue
-            .set_property("max-size-buffers", &0u32)
-            .unwrap();
-        state
-            .transcriber_queue
-            .set_property("max-size-time", &0u64)
-            .unwrap();
+            .set_property("max-size-buffers", 0u32);
+        state.transcriber_queue.set_property("max-size-time", 0u64);
 
         state.internal_bin.add(&state.transcription_bin)?;
 
-        state.textwrap.set_property("lines", &2u32).unwrap();
+        state.textwrap.set_property("lines", 2u32);
 
         state.transcription_bin.set_locked_state(true);
 
@@ -208,8 +195,7 @@ impl TranscriberBin {
 
         state
             .cccombiner
-            .set_property("latency", &(100 * gst::ClockTime::MSECOND))
-            .unwrap();
+            .set_property("latency", 100 * gst::ClockTime::MSECOND);
 
         self.audio_sinkpad
             .set_target(Some(&state.internal_bin.static_pad("audio_sink").unwrap()))?;
@@ -234,21 +220,18 @@ impl TranscriberBin {
 
         s.set("framerate", &state.framerate.unwrap());
 
-        state.cccapsfilter.set_property("caps", &cc_caps).unwrap();
+        state.cccapsfilter.set_property("caps", &cc_caps);
 
         let max_size_time = settings.latency + settings.accumulate_time;
 
         for queue in &[&state.audio_queue_passthrough, &state.video_queue] {
-            queue.set_property("max-size-bytes", &0u32).unwrap();
-            queue.set_property("max-size-buffers", &0u32).unwrap();
-            queue.set_property("max-size-time", &max_size_time).unwrap();
+            queue.set_property("max-size-bytes", 0u32);
+            queue.set_property("max-size-buffers", 0u32);
+            queue.set_property("max-size-time", max_size_time);
         }
 
         let latency_ms = settings.latency.mseconds() as u32;
-        state
-            .transcriber
-            .set_property("latency", &latency_ms)
-            .unwrap();
+        state.transcriber.set_property("latency", latency_ms);
 
         if !settings.passthrough {
             let audio_tee_pad = state.audio_tee.request_pad_simple("src_%u").unwrap();
@@ -349,20 +332,16 @@ impl TranscriberBin {
 
         gst_debug!(CAT, obj: element, "setting CC mode {:?}", mode);
 
-        state.tttocea608.set_property("mode", &mode).unwrap();
+        state.tttocea608.set_property("mode", mode);
 
         if mode.is_rollup() {
-            state
-                .textwrap
-                .set_property("accumulate-time", &0u64)
-                .unwrap();
+            state.textwrap.set_property("accumulate-time", 0u64);
         } else {
             let accumulate_time = self.settings.lock().unwrap().accumulate_time;
 
             state
                 .textwrap
-                .set_property("accumulate-time", &accumulate_time)
-                .unwrap();
+                .set_property("accumulate-time", accumulate_time);
         }
     }
 
@@ -398,6 +377,7 @@ impl TranscriberBin {
         Ok(())
     }
 
+    #[allow(clippy::single_match)]
     fn src_query(
         &self,
         pad: &gst::Pad,
@@ -474,6 +454,7 @@ impl TranscriberBin {
         })
     }
 
+    #[allow(clippy::single_match)]
     fn video_sink_event(
         &self,
         pad: &gst::Pad,
@@ -574,14 +555,14 @@ impl ObjectImpl for TranscriberBin {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
             vec![
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "passthrough",
                     "Passthrough",
                     "Whether transcription should occur",
                     DEFAULT_PASSTHROUGH,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
                 ),
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "latency",
                     "Latency",
                     "Amount of milliseconds to allow the transcriber",
@@ -590,7 +571,7 @@ impl ObjectImpl for TranscriberBin {
                     DEFAULT_LATENCY.mseconds() as u32,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "accumulate-time",
                     "accumulate-time",
                     "Cut-off time for textwrap accumulation, in milliseconds (0=do not accumulate). \
@@ -600,7 +581,7 @@ impl ObjectImpl for TranscriberBin {
                     DEFAULT_ACCUMULATE.mseconds() as u32,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_enum(
+                glib::ParamSpecEnum::new(
                     "mode",
                     "Mode",
                     "Which closed caption mode to operate in",
@@ -608,14 +589,14 @@ impl ObjectImpl for TranscriberBin {
                     DEFAULT_MODE as i32,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
                 ),
-                glib::ParamSpec::new_boxed(
+                glib::ParamSpecBoxed::new(
                     "cc-caps",
                     "Closed Caption caps",
                     "The expected format of the closed captions",
                     gst::Caps::static_type(),
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_object(
+                glib::ParamSpecObject::new(
                     "transcriber",
                     "Transcriber",
                     "The transcriber element to use",
@@ -756,6 +737,8 @@ impl ObjectImpl for TranscriberBin {
     }
 }
 
+impl GstObjectImpl for TranscriberBin {}
+
 impl ElementImpl for TranscriberBin {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -772,7 +755,7 @@ impl ElementImpl for TranscriberBin {
 
     fn pad_templates() -> &'static [gst::PadTemplate] {
         static PAD_TEMPLATES: Lazy<Vec<gst::PadTemplate>> = Lazy::new(|| {
-            let caps = gst::Caps::new_simple("video/x-raw", &[]);
+            let caps = gst::Caps::builder("video/x-raw").build();
             let video_src_pad_template = gst::PadTemplate::new(
                 "src_video",
                 gst::PadDirection::Src,
@@ -788,7 +771,7 @@ impl ElementImpl for TranscriberBin {
             )
             .unwrap();
 
-            let caps = gst::Caps::new_simple("audio/x-raw", &[]);
+            let caps = gst::Caps::builder("audio/x-raw").build();
             let audio_src_pad_template = gst::PadTemplate::new(
                 "src_audio",
                 gst::PadDirection::Src,
@@ -815,6 +798,7 @@ impl ElementImpl for TranscriberBin {
         PAD_TEMPLATES.as_ref()
     }
 
+    #[allow(clippy::single_match)]
     fn change_state(
         &self,
         element: &Self::Type,
@@ -851,4 +835,42 @@ impl ElementImpl for TranscriberBin {
     }
 }
 
-impl BinImpl for TranscriberBin {}
+impl BinImpl for TranscriberBin {
+    fn handle_message(&self, bin: &Self::Type, msg: gst::Message) {
+        use gst::MessageView;
+
+        match msg.view() {
+            MessageView::Error(ref m) => {
+                /* We must have a state here */
+                let s = self.state.lock().unwrap();
+
+                if let Some(state) = s.as_ref() {
+                    if msg.src().as_ref() == Some(state.transcriber.upcast_ref()) {
+                        gst_error!(
+                            CAT,
+                            obj: bin,
+                            "Transcriber has posted an error ({:?}), going back to passthrough",
+                            m
+                        );
+                        drop(s);
+                        let mut settings = self.settings.lock().unwrap();
+                        settings.passthrough = true;
+                        bin.notify("passthrough");
+                        drop(settings);
+                        bin.call_async(move |bin| {
+                            let thiz = TranscriberBin::from_instance(bin);
+                            thiz.block_and_update(bin, true);
+                        });
+                    } else {
+                        drop(s);
+                        self.parent_handle_message(bin, msg);
+                    }
+                } else {
+                    drop(s);
+                    self.parent_handle_message(bin, msg);
+                }
+            }
+            _ => self.parent_handle_message(bin, msg),
+        }
+    }
+}
